__author__ = 'Derek Payton <derek.payton@gmail.com>'
__copyright__ = 'Copyright (c) Derek Payton'
__description__ = 'Render inline CSS and JS in Django forms media.'
__version__ = '0.1.0'
__license__ = 'MIT License'

try:
    import django
except ImportError:
    # Django not importable, we're probably pip installing
    pass
else:
    from .media import CSS, JS



import embedded_media as emb
from django.forms import Form, CharField, TextInput, Media
from django.test import TestCase


class EmbeddedMediaTest(TestCase):
    def test_css(self):
        ## CSS rendering
        css = emb.CSS('.mywidget { display: none; }')
        self.assertHTMLEqual(css.render('all'),
            '<style type="text/css" media="all">.mywidget { display: none; }</style>')

    def test_js(self):
        ## JS rendering
        js = emb.JS('init_mywidget();')
        self.assertHTMLEqual(js.render(),
            '<script type="text/javascript">init_mywidget();</script>')

    def test_media(self):
        ## Embedded JavaScript and CSS
        class MyWidget(TextInput):
            class Media:
                css = {'all': (emb.CSS('.mywidget { display: none; }'),)}
                js = (emb.JS('init_mywidget();'),)

        w = MyWidget()
        self.assertHTMLEqual(str(w.media['css']),
            '<style type="text/css" media="all">.mywidget { display: none; }</style>')
        self.assertHTMLEqual(str(w.media['js']),
            '<script type="text/javascript">init_mywidget();</script>')

    def test_property(self):
        ## Embedded JavaScript and CSS as a media property
        class MyWidget(TextInput):
            def _media(self):
                return Media(
                    css={'all': (emb.CSS('.mywidget { display: none; }'),)},
                    js=(emb.JS('init_mywidget();'),)
                )
            media = property(_media)

        w = MyWidget()
        self.assertHTMLEqual(str(w.media['css']),
            '<style type="text/css" media="all">.mywidget { display: none; }</style>')
        self.assertHTMLEqual(str(w.media['js']),
            '<script type="text/javascript">init_mywidget();</script>')

    def test_mutiple(self):
        ## Multiple instances of embedded media should only be rendered once
        class MyWidget(TextInput):
            def _media(self):
                return Media(
                    css={'all': (emb.CSS('.mywidget { display: none; }'),)},
                    js=(emb.JS('init_mywidget();'),)
                )
            media = property(_media)

        class MyForm(Form):
            field1 = CharField(widget=MyWidget)
            field2 = CharField(widget=MyWidget)

        f = MyForm()
        self.assertHTMLEqual(str(f.media), """
            <style type="text/css" media="all">.mywidget { display: none; }</style>
            <script type="text/javascript">init_mywidget();</script>
        """)



#!/usr/bin/env python

import embedded_media as emb

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

setup(
    name='django-embedded-media',
    version=emb.__version__,
    description=emb.__description__,
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    keywords='django media',
    maintainer='Derek Payton',
    maintainer_email='derek.payton@gmail.com',
    url='https://github.com/dmpayton/django-embedded-media',
    download_url='https://github.com/dmpayton/django-embedded-media/tarball/v%s' % emb.__version__,
    license=emb.__license__,
    include_package_data=True,
    packages=find_packages(exclude=('tests',)),
    test_suite='tests.setuptest.SetupTestSuite',
    tests_require=(
        'coverage',
        'django>=1.4',
        'pep8',
      ),
    zip_safe=False,
    )



from django.forms.widgets import Media
from django.utils.html import conditional_escape, format_html
from django.utils.safestring import mark_safe


__all__ = ('CSS', 'JS')


class EmbeddedMedia(object):
    """
    Base class for adding embedded CSS and JS to a Media class.
    """
    def __init__(self, content, **attrs):
        self.content = content
        self.attrs = attrs

    def __eq__(self, other):
        return (
            type(self) is type(other) and
            self.content == other.content
        )

    def build_attrs(self):
        attrs = [(k, conditional_escape(v)) for k, v in self.attrs.iteritems()]
        attrs = ' '.join(['{0}="{1}"'.format(*x) for x in attrs])
        return attrs


class CSS(EmbeddedMedia):
    def render(self, medium=None):
        medium = medium or 'all'
        attrs = self.build_attrs()
        html = '<style type="text/css" media="{{0}}" {0}>{{1}}</style>'.format(attrs)
        return format_html(html, medium, mark_safe(self.content))


class JS(EmbeddedMedia):
    def render(self):
        attrs = self.build_attrs()
        html = '<script type="text/javascript" {0}>{{0}}</script>'.format(attrs)
        return format_html(html, mark_safe(self.content))


def render_css(self):
    # To keep rendering order consistent, we can't just iterate over items().
    # We need to sort the keys, and iterate over the sorted list.
    html = []
    media = sorted(self._css.keys())
    for medium in media:
        for style in self._css[medium]:
            if hasattr(style, 'render'):
                html.append(style.render(medium))
            else:
                html.append(format_html('<link href="{0}" type="text/css" media="{1}" rel="stylesheet" />', self.absolute_path(style), medium))
    return html

Media.render_css = render_css


def render_js(self):
    html = []
    for script in self._js:
        if hasattr(script, 'render'):
            html.append(script.render())
        else:
            html.append(format_html('<script type="text/javascript" src="{0}"></script>', self.absolute_path(script)))
    return html

Media.render_js = render_js
